// This function will be initialized with template variables from the HTML
function initializeLogin(errorMessage, loginUrl) {
    console.log("initializeLogin called with:", { errorMessage, loginUrl });
    
    const {
        AppBar, Toolbar, Typography, Button, TextField,
        Paper, Container, Box, Card, CardContent,
        IconButton, InputAdornment
    } = MaterialUI;
    
    // Check if ThemeWrapper exists
    console.log("ThemeWrapper exists:", typeof ThemeWrapper !== 'undefined');
    
    function LoginApp() {
        console.log("LoginApp component rendering");
        const [showPassword, setShowPassword] = React.useState(false);
        const [password, setPassword] = React.useState('');

        const handleSubmit = (e) => {
            e.preventDefault();
            e.target.submit();
        };

        // Create the login UI without ThemeWrapper first
        const loginUI = (
            <Box sx={{ flexGrow: 1 }}>
                <AppBar position="static" elevation={4}>
                    <Toolbar>
                        <Typography variant="h6" component="div" sx={{ flexGrow: 1 }}>
                            Anges AI
                        </Typography>
                        {/* The theme toggle button will receive props from ThemeWrapper */}
                        <IconButton 
                            color="inherit" 
                            sx={{ ml: 1 }}
                            title="Toggle light/dark theme"
                        >
                            <span className="material-icons">
                                dark_mode
                            </span>
                        </IconButton>
                    </Toolbar>
                </AppBar>

                <Container maxWidth="sm" sx={{ mt: 8 }}>
                    <Card elevation={4}>
                        <CardContent sx={{ p: 4 }}>
                            <Typography variant="h4" component="h1" gutterBottom align="center">
                                Welcome
                            </Typography>

                            {errorMessage && (
                                <Box className="error-message">
                                    <span className="material-icons">error</span>
                                    {errorMessage}
                                </Box>
                            )}

                            <form method="POST" action={loginUrl} onSubmit={handleSubmit}>
                                <TextField
                                    fullWidth
                                    type={showPassword ? 'text' : 'password'}
                                    name="password"
                                    label="Enter password"
                                    variant="outlined"
                                    value={password}
                                    onChange={(e) => setPassword(e.target.value)}
                                    sx={{ mb: 3 }}
                                    InputProps={{
                                        endAdornment: (
                                            <InputAdornment position="end">
                                                <IconButton
                                                    onClick={() => setShowPassword(!showPassword)}
                                                    edge="end"
                                                >
                                                    <span className="material-icons">
                                                        {showPassword ? 'visibility_off' : 'visibility'}
                                                    </span>
                                                </IconButton>
                                            </InputAdornment>
                                        ),
                                    }}
                                />

                                <Button
                                    type="submit"
                                    fullWidth
                                    variant="contained"
                                    size="large"
                                    startIcon={<span className="material-icons">login</span>}
                                >
                                    Login
                                </Button>
                            </form>
                        </CardContent>
                    </Card>
                </Container>
            </Box>
        );

        try {
            if (typeof ThemeWrapper === 'undefined') {
                console.error("ThemeWrapper is not defined!");
                // Fallback to original design if ThemeWrapper isn't available
                return loginUI;
            }
            
            // Enhanced version of loginUI that uses the ThemeWrapper
            // The ThemeWrapper component will clone the loginUI element and add props to it
            const EnhancedLoginUI = React.forwardRef((props, ref) => {
                console.log("EnhancedLoginUI rendering with props:", props);
                
                // Create a new version of loginUI with the theme toggle button updated
                return React.cloneElement(loginUI, {}, 
                    React.cloneElement(loginUI.props.children[0], {}, 
                        React.cloneElement(loginUI.props.children[0].props.children, {}, 
                            // Keep the Typography component
                            loginUI.props.children[0].props.children.props.children[0],
                            // Replace the IconButton with one that uses the theme props
                            React.cloneElement(
                                loginUI.props.children[0].props.children.props.children[1], 
                                { 
                                    onClick: props.toggleColorMode 
                                }, 
                                React.cloneElement(
                                    loginUI.props.children[0].props.children.props.children[1].props.children, 
                                    {}, 
                                    props.currentTheme === 'dark' ? 'light_mode' : 'dark_mode'
                                )
                            )
                        )
                    ),
                    // Keep the Container component unchanged
                    loginUI.props.children[1]
                );
            });
            
            // Now wrap the enhanced login UI with ThemeWrapper
            return (
                <ThemeWrapper>
                    <EnhancedLoginUI />
                </ThemeWrapper>
            );
        } catch (error) {
            console.error("Error rendering LoginApp:", error);
            // Display error in the UI for debugging
            return (
                <Box sx={{ p: 4, color: 'error.main' }}>
                    <Typography variant="h5">Error Rendering Login</Typography>
                    <pre>{error.toString()}</pre>
                    <pre>{error.stack}</pre>
                </Box>
            );
        }
    }

    try {
        console.log("Attempting to render LoginApp");
        ReactDOM.render(<LoginApp />, document.getElementById('root'));
        console.log("LoginApp rendered successfully");
    } catch (error) {
        console.error("Error during ReactDOM.render:", error);
        // Display error on the page for debugging
        document.getElementById('root').innerHTML = `
            <div style="color: red; padding: 20px;">
                <h2>Error Rendering Login Page</h2>
                <p>${error.toString()}</p>
                <pre>${error.stack}</pre>
            </div>
        `;
    }
}
